function create_user() {
    # create_user <username> <uid>
    username=$1
    uid=$2

    user_exists=$(check_by_user ${username})
    uid_exists=$(check_by_uid ${uid})

    if [ "${user_exists}" == "true" ] || [ "${uid_exists}" == "true" ]; then
        user_match_uid=$(match_user_uid ${username} ${uid})
        if [ "${user_match_uid}" == "true" ]; then
            echo "The username already exists"
            return 0
        else
            echo "Username or UID already exists, but are not ours."
            return 1
        fi
    fi

    echo "Creating username: ${username}"
    useradd ${username} -m --uid ${uid}
    /opt/confluent/bin/nodeshell all useradd ${username}  --uid ${uid}
}

function check_by_user() {
    # check_by_user <username>
    status=$(getent passwd $1 | cut -f1 -d":")
    if [ "$status" == "$1" ]; then
        echo "true"
    else
        echo "false"
    fi
}

function check_by_uid() {
    # check_by_uid <uid>
    status=$(getent passwd $1 | cut -f3 -d":")
    if [ "$status" == "$1" ]; then
        echo "true"
    else
        echo "false"
    fi
}

function match_user_uid() {
    # match_user_uid <username> <uid>
    existing_user=$(getent passwd $1 | cut -f1 -d":")
    existing_user_uid=$(getent passwd $1 | cut -f3 -d":")
    if [ "$existing_user" == "$1" ] && [ "${existing_user_uid}" == "$2" ]; then
        echo "true"
    else
        echo "false"
    fi
}

function slurm_mpi_test() {
    # slurm_mpi_test <username> <uid> <user_home_dir> <job_file_location>
    username=$1
    uid=$2
    job_file_location=$4
    user_home_dir=$3
    create_user ${username} ${uid}
    mkdir -p /${user_home_dir}/${username}/slurm_mpi_test
    echo "Compiling MPI program"
    source /etc/profile.d/lmod.sh
    /opt/ohpc/pub/mpi/openmpi4-gnu9/4.1.1/bin/mpicc -O3 /opt/ohpc/pub/examples/mpi/hello.c -o /${user_home_dir}/${username}/slurm_mpi_test/mpi_program
    \cp ${job_file_location} /${user_home_dir}/${username}/slurm_mpi_test/mpi_test.slurm
    chown -R ${username} /${user_home_dir}/${username}/slurm_mpi_test/
    echo "Starting MPI Slurm job..."
    job=$(su ${username} -l -c "cd /${user_home_dir}/${username}/slurm_mpi_test/ && sbatch mpi_test.slurm")
    job_id=${job##* }

    sleep 10

    job_status=$(scontrol show job ${job_id} | grep JobState | cut -d" " -f4 | cut -d"=" -f2)

    if [ "${job_status}" != "COMPLETED" ]; then
        echo "JobId: ${job_id} - FAIL"
        return 1
    else
        echo "JobId: ${job_id} - SUCCESS"
        return 0
    fi
}

function cleanup() {
    # cleanup <username>
    username=$1
    userdel ${username}
    /opt/confluent/bin/nodeshell all userdel ${username}
}

test_user="mpitestuser"
test_user_uid="65540"
slurm_mpi_test ${test_user} ${test_user_uid} ${storage_user_home_dir} ${job_file_location}
cleanup ${test_user}
